__author__ = 'Eric'

from modem_connection import ModemConnection
from iridium_connection import IridiumConnection
from serial_connection import SerialConnection
from sbd_connection import SBDEmailConnection
from tcp_connection import TcpConnection
